-- ==================================
-- panel7Box.lua
-- Author of panel - Gennady Lygin a.k.a Croc
-- Author of tools - Vincent Giangiulio a.k.a Vincent
-- Integration by Gennady Lygin a.k.a Croc 17.07.2016
-- Edit by Patrick Ziegler a.k.a. Gironde 12-17.08.2018
-- Edit by Gennady Lygin a.k.a Croc 13.08.2018
-- ==================================
require "locale"
require "closeButton"
----------------------------------------------
-- Set initial values
----------------------------------------------
local newwidth, newheight
local gn=5
local gg=gn-1
local panel7FrameWidth, panel7FrameHeight = 186, 29*gg+57
----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------
panel7Box = CXBox:new() 
----------------------------------------------
panel7Frame = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0,0.5,0.5,.8})
    :fillcolor(csetbofill) 
    :textfont(normalfont)   
    :textcolor({0.2,0.8,1,1})
    :textpos("center")
    :movetext(0, 5)
    :active(false)
    :text(_("Time Machine")) 
    :movable(true)
    :clickable(false)    
    :visible(false)
    :attach(screenBox, (width-panel7FrameWidth)/2, height-panel7FrameHeight-2, (width-panel7FrameWidth)/2, 2)

panel7Frame.Customdraw = function(this)  
       if newwidth ~= width or newheight ~= height then
       newwidth, newheight = width, height;

       if width >=1280 then               
           panel7Frame:attach(screenBox, (width-panel7FrameWidth)/2+541, 3, (width-panel7FrameWidth)/2-541, height-panel7FrameHeight-3) 
        else
          if width>=900  then
              panel7Frame:attach(screenBox, width-panel7FrameWidth-35+30, 180, 35-30, height-panel7FrameHeight-180) 
          else
              panel7Frame:attach(screenBox, width-panel7FrameWidth-35+30, 357, 35-30, height-panel7FrameHeight-357)
          end
       end
    end
end;

closepanel7Button = CXBox:new()
        :init(0, 0, 0, 0) 
        :textfont(smallfont)
        :textcolor({1,0,0,0.5})
        :textpos("left")
        :movetext(1, 12)
        :text("x")
        :movable(false)
        :active(true)
        :attach(panel7Frame, 172, 160, 2, 2)

closepanel7Button.Action = (function()
    return
        function()   
            display_panel7 = not display_panel7
            if panel7Frame.Visible then                   
                 panel7Frame.Visible = false
                 panelAll27Check.Text = ""    
            end             
        end
    end) ()

--========================
-- timeBox.lua
--========================
-- require "locale";
-- require "closeButton";
----------------------------------------------
-- Load images.
----------------------------------------------
timeRevImage = celestia:loadtexture("../images/time_rev_button.png")
time1xImage = celestia:loadtexture("../images/time_1x_button.png")
timeDiv2Image = celestia:loadtexture("../images/time_div2_button.png")
timeDiv10Image = celestia:loadtexture("../images/time_div10_button.png")
timeMult2Image = celestia:loadtexture("../images/time_mult2_button.png")
timeMult10Image = celestia:loadtexture("../images/time_mult10_button.png")
timeCurImage = celestia:loadtexture("../images/time_cur_button.png")

----------------------------------------------Разделительная точка
-- Define local functions
----------------------------------------------
local setDateColorsOff = function()
    daycolor = cbutextoff
    monthcolor = cbutextoff
    yearcolor = cbutextoff
    hourcolor = cbutextoff
    minutecolor = cbutextoff
    secondcolor = cbutextoff
	centiseccolor = color2 -- by Croc --cbutextoff -- *******  added by Gironde 17/08/2018
    timezonecolor = cbutextoff
end

local setCurrentTime = function()
    local UTCtime = os.date("!*t")
    celestia:settime(celestia:utctotdb(UTCtime.year, UTCtime.month, UTCtime.day, UTCtime.hour, UTCtime.min, UTCtime.sec))
end

local setTime = function()
    celestia:settime(celestia:utctotdb(date.year, date.month, date.day, date.hour, date.minute, date.seconds))
end

----------------------------------------------
-- Set initial values
----------------------------------------------
-- Define a limit value for timescale.
-- This shouldn't be necessary but timerate strangely
-- keeps on increasing when time reaches limit values...
limitTimerate = 1e15

-- Define max/min values for time to avoid bugs with Julian days...
maxTime = 730486721060.00073; -- 2000000000 Jan 01 12:00:00 UTC
minTime = -730498278941.99951; -- -2000000000 Jan 01 12:00:00 UTC

setDateColorsOff()
daycolor = cbutexton
toset = "day"

-- Get displayUTCtime value from 'config.lua'.
displayUTCtime = not(show_local_time)

-- Get timezone_cfg value from 'config.lua'.
timezone_cfg = time_zone

timescale = celestia:gettimescale()
paused = celestia:ispaused()

-- =================================================
-- Размеры панели коррекции времени: ширина / высота; размер пиктограмм
-- =================================================

setTimeFrameW, setTimeFrameH = 189, 70

setTimescaleButtonW = (boxWidth - 16) / 7

-- =================================================
-- Set up and Draw the boxes
-- =================================================
------------------------------------------------------------------------
-- Общая рамка для Машины Времени (для часов и кнопок)
------------------------------------------------------------------------

timeBox = CXBox:new()
      :init(0, 0, 0, 0)
      :movable(true)
      :attach(panel7Frame, 2, 29*(gg-0)-12, 2, 20)

timeButtonFrame = CXBox:new()
      :init(0, 0, 0, 0)
      :movable(true)
      :attach(timeBox, 0, 0, 0, 0)
--------------------------------
-- Рамка для дисплея
--------------------------------
timeFrame = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor({0.2,0.5,1,.3}) 
    :movable(false)
    :attach(timeBox, 2, 20, 2, 0)

timeFrame.Customdraw =
    function(this)
        local time = celestia:gettime()
        local timescale = celestia:gettimescale()
        -- Fix max/min time values.
        if time <= minTime then
            celestia:settime(minTime)
            celestia:settimescale(0)
        elseif time >= maxTime then
            celestia:settime(maxTime)
            celestia:settimescale(0)
        end
        ----------------------------------------------------------
        -- Convert Celestia's internal TDB time to UTC date table.
        -- Преобразование внутреннего времени TDB Celestia в UTC...
        ----------------------------------------------------------
        date = celestia:tdbtoutc(time)
        ----------------------------------------------------------
        -- Get OS time in seconds from the UTC date table.
        -- Получить OS время в секундах, из UTC таблицы "дата" .
        ----------------------------------------------------------

        local systime = os.time{year=date.year, month=date.month, day=date.day, hour=date.hour, min=date.minute, sec=date.seconds}

        if systime then
            -- Get the local date table from OS.
            local syslocdate  = os.date("*t", systime + 3600)
            -- Get the UTC date from OS.
            local sysUTCdate = os.date("!*t", systime + 3600)
            -- Compute the system locale Time Zone.
            local daydiff = syslocdate.day - sysUTCdate.day
            local hourdiff
            if daydiff == 0 then
                hourdiff = 0
            elseif daydiff > 1 or daydiff == -1 then
                hourdiff = -24
            else
                hourdiff = 24
            end
            timezone = hourdiff + syslocdate.hour - sysUTCdate.hour + (syslocdate.min - sysUTCdate.min) / 60

            ---------------------------------------------------------------------------------------------------
            -- HACK: Get the local date table from the 'modified' TDB time using Celestia's 'tdbtoutc' function.
            -- HACK: Получить локальную таблицу дату из "модифицированной" TDB времени с использованием функции Celestia  "tdbtoutc".
            ---------------------------------------------------------------------------------------------------

            locdate = celestia:tdbtoutc(time + timezone/24)
        end
        -----------------------------------------------------------
        -- Set Time Zone string
        -- Установить строку часовой пояс
        -----------------------------------------------------------

        if displayUTCtime or not(systime) then
            timezone_str = "   UTC"
            day_str = math.abs(date.day) 
			month_str = monthAbbr[math.abs(date.month)] 
			year_str = date.year
            hour_str = format:twodigits(math.abs(date.hour)) 
			min_str = format:twodigits(math.abs(date.minute))
            sec_str = format:twodigits(math.floor(math.abs(date.seconds)))
            centisec_str = format:twodigits((math.abs(date.seconds) - math.floor(math.abs(date.seconds)))*100) -- *******  added by Gironde 17/08/2018
			
        elseif type(time_zone) == "number" then
            ------------------------------------------------------
            -- A Time Zone value has been set in config.lua.
            -- Значение часового пояса было установлено в config.lua.
            ------------------------------------------------------

            date_cfg = celestia:tdbtoutc(time + timezone_cfg/24)
            timezone_str = "GMT"..string.format("%+0.0f", math_sign(timezone_cfg) * math.floor(math.abs(timezone_cfg)))
            if math.floor(timezone_cfg) ~= timezone_cfg then
                ----------------------------------------------
                -- Timezone is an non-integer number of hours.
                -- Часовой пояс не целое количество часов
                ----------------------------------------------
                timezone_str = timezone_str.._("h")..(math.abs(timezone_cfg) - math.floor(math.abs(timezone_cfg))) * 60
            end
            day_str = date_cfg.day 
			month_str = monthAbbr[date_cfg.month] 
			year_str = date_cfg.year
            hour_str = format:twodigits(date_cfg.hour)
			min_str = format:twodigits(date_cfg.minute) 
			sec_str = format:twodigits(math.floor(date_cfg.seconds))
            centisec_str = format:twodigits(math.floor((date_cfg.seconds - (math.floor(math.abs(date_cfg.seconds))))*100)) -- *******  added by Gironde 17/08/2018			
        else
            -----------------------------------
            -- Use system locale Time Zone.
            -- Использование языка системы часовых поясов.
            -----------------------------------

            timezone_str = "GMT"..string.format("%+0.0f", math_sign(timezone) * math.floor(math.abs(timezone)))
            if math.floor(timezone) ~= timezone then
                -------------------------------------------------
                -- Timezone is an non-integer number of hours.
                -- Часовой пояс не целое количество часов
                -------------------------------------------------
                timezone_str = timezone_str.._("h")..(math.abs(timezone)-math.floor(math.abs(timezone)))*60
            end
            day_str = locdate.day 
			month_str = monthAbbr[locdate.month] 
			year_str = locdate.year
            hour_str = format:twodigits(locdate.hour) 
			min_str = format:twodigits(locdate.minute) 
			sec_str = format:twodigits(math.floor(locdate.seconds))
            centisec_str = format:twodigits(math.floor((locdate.seconds - (math.floor(math.abs(locdate.seconds))))*100)) -- *******  added by Gironde 17/08/2018
        end
       ---------------------------------------------------------------
       -- Определение дня недели:
       ---------------------------------------------------------------
	   
	   local datetime = os.time({ year=date.year, month = date.month, day = date.day, hour = date.hour+4, minute = date.minute}) 
       local weekday = os.date("%w",datetime)


        -------------------------------------------------------------------------------------
        -- Set Date string. Use date format from the locale/lang/lang.lua file.
        -- Определить дату (строку). Используйте формат даты из locale/ Lang / lang.lua файл.
        -------------------------------------------------------------------------------------

        if date_format == "year/month/day" then
            ------------------------
            -- Big endian format.  
            -- Формат ГГГГ-ММ-ДД          
            ------------------------
            date_str = year_str..' '..month_str..' '..day_str
        elseif date_format == "month/day/year" then
            --------------------------
            -- Middle endian format.
            -- Формат ММ-ДД-ГГГГ
            --------------------------
            date_str = month_str..' '..day_str..' '..year_str
        else
            ----------------------------------
            -- Little endian format (default).
            -- Формат ДД-ММ-ГГГГ
            ----------------------------------

            date_str = day_str..' '..month_str..' '..year_str
        end
        ---------------------------------------------------------------------------------
        -- Set Time string.
        -- Определить время (строку).
        ---------------------------------------------------------------------------------

        time_str = hour_str..' : '..min_str..' : '..sec_str

        ---------------------------------------------------------------------------------
        -- Display Date, Time, Timezone.
        -- Отобразить дату, время, часовой пояс.
        ---------------------------------------------------------------------------------

        textlayout:setfont(normalfont)
        textlayout:setlinespacing(normalfont:getheight())
        ------------------------------------------------ date_str
        textlayout:setfontcolor({0,1,0,0.9})   
        textlayout:setpos(this.lb+5, this.tb-11)
        textlayout:println(date_str);
        ------------------------------------------------ time_str        
        textlayout:setpos(this.lb+115, this.tb-11)

        if year_str <100000 then
           textlayout:println(time_str)
        end
        ------------------------------------------------ wd  (weekday) --edit Gironde 22-11-2017

		textlayout:setfontcolor({1,0,0,0.9})
        textlayout:setpos(this.lb+5, this.tb-23)
        if weekday == "0" then
           textlayout:println (_"Sunday")
        elseif weekday == "1" then
           textlayout:println (_"Monday")
        elseif weekday == "2" then
           textlayout:println (_"Tuesday")
        elseif weekday == "3" then
           textlayout:println (_"Wednesday")
        elseif weekday == "4" then
           textlayout:println (_"Thursday")
        elseif weekday == "5" then
           textlayout:println (_"Friday")
        elseif weekday == "6" then
           textlayout:println (_"Saturday")
        end
		
		------------------------------------------------ timezone_str
        -- Set Timerate string.
        -- Определить Скорость Времени (строка) - Множитель
        ---------------------------------------------------------------

        if celestia:ispaused() then
            ---------------------------------------------------
            -- Time is paused
            ---------------------------------------------------

            timescale_str = ""
            
            textlayout:setfontcolor(ctextalert)          
            textlayout:setpos(this.lb+7+normalfont:getwidth(_("Rate:"))+65-30, this.tb+7-30)
            textlayout:println(_("(paused)"))

            textlayout:setfontcolor(ctext)
        elseif math.abs(timescale) <= 1e-15 then

            ---------------------------------------------------
            -- Time is stopped.
            ---------------------------------------------------
            timescale_str = _("time stopped")
        else
            timescale_str = string.format("%g", timescale).."x"
        end

        ------------------------------------------------------
        -- Display Timerate
        -- Отобразить Множитель времени. --  Rate
        ------------------------------------------------------
        textlayout:setfontcolor(cbutextoff) 
        textlayout:setpos(this.lb+90, this.tb-23)
        textlayout:println(timescale_str)
        end;

-- ===========================
--  Кнопка "Коррекция времени"  "Set Time" 
-- ===========================

setTimeButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0,0.5,0.5,0.8})
    :fillcolor(cFrameFill)    
    :textfont(normalfont)
    :textcolor({1,0,0,0.7})   
    :textpos("center")
    :movetext(0, 8)
    :text(_("TM"))
    :movable(false)
    :active(true)
    :attach(panel7Frame, 4, 156, 160, 3) 

setTimeButton.Action = (function()
    return
        function()
            setTimeFrame.Visible = not(setTimeFrame.Visible)
            if not(setTimeFrame.Visible) then
                setTimeButton.Bordercolor = cbubordoff
                setTimeButton.Fillcolor = nil
            else
                setTimeButton.Bordercolor = cbubordon
                setTimeButton.Fillcolor = {cbubordon[1]*2/3, cbubordon[2]*2/3, cbubordon[3]*2/3, cbubordon[4]/2}
            end
        end
    end) ()

---------------------------------------
-- Панель "Коррекция времени"
----------------------------------------
setTimeFrame = CXBox:new()
    :bordercolor(cbubordoff) 
    :fillcolor(csetbofill)
    :visible(false)
    :active(false)  
    :attach(setTimeButton, -15, 20, -171, -90)

setTimeFrame.Customdraw =
    function(this)
        -- We add the closeButton on the second tick
        -- when bounds are getting updated.
        if setTime_firstTickDone then
            if not(setTimeCloseButtonDone) then
                setTimeCloseButtonDone = true;
                makeCloseButton(setTimeCloseButton, setTimeFrame, setTimeButton)
            end
        else
            setTime_firstTickDone = true
        end
    end

-- ========================= Панель корректировки времени
------------------------------------------------- 
-- Панель корректировки времени 
-- Бокс текст (заголовок)
-------------------------------------------------
text0Box = CXBox:new()
   -- :bordercolor({0.2, 0.2 , 0.6, 0.6})
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 5, 85, 5, 4)

text0Box.Customdraw =
    function(this)
        textlayout:setfontcolor({0,1,1,0.5})
        textlayout:setpos(this.lb+0+30, this.tb-10+1)
        textlayout:println(_("Teleportation. Arrival:")) 
    end
	
------------------------------------------------- 
-- Панель корректировки времени 
-- Бокс текст (Дата)
-------------------------------------------------	
text1Box = CXBox:new()
   -- :bordercolor({0.2, 0.2 , 0.6, 0.6})
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 5, 58, 140, 20)

text1Box.Customdraw =
    function(this)
        textlayout:setfontcolor({0,1,1,0.5})
        textlayout:setpos(this.lb+0, this.tb-11)
        textlayout:println(_("Date:")) 
    end	
------------------------------------------------- 
-- Панель корректировки времени 
-- Бокс день - dayBox
-------------------------------------------------
dayBox = CXBox:new()
    :bordercolor({0.2, 0.2 , 0.6, 0.6})   
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 37, 58, 130, 20)

dayBox.Customdraw =
    function(this)
        textlayout:setfontcolor(daycolor)
        textlayout:printinbox(day_str, dayBox, 0, -3)
    end

dayBox.Action = (function()
    return
        function()
            toset = "day"
            setDateColorsOff()
            daycolor = cbutexton
        end
    end) ()

------------------------------------------------- 
-- Панель корректировки времени 
-- Бокс месяц - monthBox
-------------------------------------------------    
monthBox = CXBox:new()
    :bordercolor({0.2, 0.2 , 0.6, 0.6})
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 59, 58, 86, 20)

monthBox.Customdraw =
    function(this)
        textlayout:setfontcolor(monthcolor)
        textlayout:printinbox(month_str, monthBox, 0, -3)
    end

monthBox.Action = (function()
    return
        function()
            toset = "month"
            setDateColorsOff()
            monthcolor = cbutexton
        end
    end) ()

------------------------------------------------- 
-- Панель корректировки времени 
-- Бокс год - yearBox
-------------------------------------------------   
yearBox = CXBox:new()
   
    :bordercolor({0.2, 0.2 , 0.6, 0.6})
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 103, 58, 46, 20)

yearBox.Customdraw =
    function(this)
        textlayout:setfontcolor(yearcolor)
		textlayout:printinbox(year_str, yearBox, 0, -3)  -- add Gironde 13.08.2018
    end

yearBox.Action = (function()
    return
        function()
            toset = "year"
            setDateColorsOff()
            yearcolor = cbutexton
        end
    end) ()

------------------------------------------------- 
-- Панель корректировки времени 
-- Бокс текст (время)
-------------------------------------------------
text2Box = CXBox:new()
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 5, 42, 140, 37)

text2Box.Customdraw =
    function(this)
        textlayout:setfontcolor({0,1,1,0.5})
        textlayout:setpos(this.lb+0, this.tb-10)
        textlayout:println(_("Time:"))      
    end

------------------------------------------------- 
-- Панель корректировки времени 
-- Бокс часов - hourBox
-------------------------------------------------
hourBox = CXBox:new()
    :bordercolor({0.2, 0.2 , 0.6, 0.6})
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 57-20, 41, 130, 37)

hourBox.Customdraw =
    function(this)
        textlayout:setfontcolor(hourcolor);      
        textlayout:printinbox(hour_str..":", hourBox, 0, -3)     
    end

hourBox.Action = (function()
    return
        function()
            toset = "hour"
            setDateColorsOff()
            hourcolor = cbutexton
        end
    end) ()

------------------------------------------------- 
-- Панель корректировки времени 
-- Бокс минут - minuteBox
-------------------------------------------------
minuteBox = CXBox:new()
    :bordercolor({0.2, 0.2 , 0.6, 0.6})
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 59, 41, 108, 37)

minuteBox.Customdraw =
    function(this)
        textlayout:setfontcolor(minutecolor)
        textlayout:printinbox(min_str..":", minuteBox, 0, -3)
    end;

minuteBox.Action = (function()
    return
        function()
            toset = "minute"
            setDateColorsOff()
            minutecolor = cbutexton
        end
    end) ()

------------------------------------------------- 
-- Панель корректировки времени 
-- Бокс секунд - secondBox
-------------------------------------------------
secondBox = CXBox:new()
    :bordercolor({0.2, 0.2 , 0.6, 0.6})
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 81, 41, 86, 37)

secondBox.Customdraw =
    function(this)
        textlayout:setfontcolor(secondcolor);
        textlayout:printinbox(sec_str..".", secondBox, 0, -3)
    end

secondBox.Action = (function()
    return
        function()
            toset = "seconds"
            setDateColorsOff()
            secondcolor = cbutexton
        end
    end) ()
	
------------------------------------------------- 
-- Панель корректировки времени 
-- Бокс доли секунд - centisecBox
-------------------------------------------------

centisecBox = CXBox:new()
    :bordercolor({0.2, 0.2 , 0.6, 0.6})  
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 105, 41, 68, 37)

centisecBox.Customdraw =
    function(this)
         textlayout:setfontcolor(centiseccolor);
	     textlayout:printinbox(centisec_str, centisecBox, 0, -3) -- by Croc 17/08/2018
    end

centisecBox.Action = (function()
   return
      function()
         toset = "centiseconds"
         setDateColorsOff()
         centiseccolor = cbutexton
         previouscolor = color4 -- added 13/08/2018 by Gironde
         previous100color = color0 -- added 13/08/2018 by Gironde
      end
   end) ()
------------------------------------------------- 
-- Панель корректировки времени 
-- "hh:mm:ss.ss"
-------------------------------------------------
text4Box = CXBox:new()
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 132, 42, 2, 37)

text4Box.Customdraw =
    function(this)
	    textlayout:setfont(smallfont)
        textlayout:setfontcolor({0,1,1,0.5})
        textlayout:setpos(this.lb-8, this.tb-10)
        textlayout:println(_("hh:mm:ss.ss"))     
    end;

-------------------------------------------------
-- Панель корректировки времени 
-- Бокс текст (часовой пояс)
-------------------------------------------------
text3Box = CXBox:new()
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 5, 7, 95, 72)

text3Box.Customdraw =
    function(this)
	    textlayout:setfont(normalfont)
        textlayout:setfontcolor({0,1,1,0.5})
        textlayout:setpos(this.lb+0, this.tb-10)
        textlayout:println(_("Time zone:"))     
    end

------------------------------------------------- 
-- Панель корректировки времени 
-- Часовой пояс - timeZoneBox
-------------------------------------------------

timeZoneBox = CXBox:new()
    :bordercolor({0.2, 0.2 , 0.6, 0.6})
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 81, 5, 46, 73)

timeZoneBox.Customdraw =
    function(this)
        textlayout:setfontcolor(timezonecolor)
        local dx = (timeZoneBox:size()-normalfont:getwidth(timezone_str))/2
        textlayout:printinbox(timezone_str, timeZoneBox, -dx+10, -3)
    end;

timeZoneBox.Action = (function()
    return
        function()
            toset = ""
            displayUTCtime = not(displayUTCtime)
            setDateColorsOff()
            timezonecolor = cbutexton
        end 
    end) ()	
	
-------------------------------------------------
-- Панель корректировки времени 
-- Юлианский календарь - jdBox
-------------------------------------------------
jdBox = CXBox:new()
    --:bordercolor({0.2, 0.2 , 0.6, 0.6})
    :textfont(normalfont)
    :textcolor({cbutextoff[1], cbutextoff[2], cbutextoff[3], cbutextoff[4]*0.6})
    :movetext(0, 12)
    :text("")
    :visible(true)
    :attach(setTimeFrame, 0, 19, 5, 59)
 
jdBox.Customdraw =
    function(this)
        local jdUTCtime = celestia:tojulianday(date.year, date.month, date.day, date.hour, date.minute, date.seconds)
    	jdBox.Text = _("JDate:")..string.format("%0.7f", jdUTCtime)
    end	
-------------------------------------------------
-- Панель корректировки времени 
-- Кнопка "предыдущее значение" - "<"
-------------------------------------------------
setTimePreviousButton = CXBox:new()
    :bordercolor(cbubordoff)
    :fillcolor(cslidebofill)
    :textfont(smallfont)
    :textcolor(cbutextoff)
    :textpos("left")
    :movetext(-2, 8+2)
    :text("<")
    :attach(setTimeFrame, 150, 58, 24, 20)
    :visible(true)
    :active(true)

setTimePreviousButton.Action = (function()
   return
      function()
         if toset ~= "" then -- added 13/08/2018 by Gironde
            if toset == "centiseconds" then
               date["seconds"] = date["seconds"]-0.01
               setTime()
            else
               date[toset] = date[toset]-1
               setTime()
            end
         end
      end
   end) ()	
-------------------------------------------------
-- Панель корректировки времени 
-- Кнопка "100 единиц назад" - "<<<"
-------------------------------------------------

setTimePrevious100Button = CXBox:new()
    :bordercolor(cbubordoff)
    :fillcolor(cslidebofill)
    :textfont(smallfont)
    :textcolor(cbutextoff)
    :textpos("left")
    :movetext(-2, 8+2)
    :text("<<<")
    :visible(true)
    :active(true)
    :attach(setTimeFrame, 150, 23, 7, 55)

setTimePrevious100Button.Action = (function()
    return
        function()
        if toset ~= "" and toset == "year" then  -- edit Gironde 13.08.2018	
            date[toset] = date[toset]-100
            setTime()
          end  -- edit Gironde 12.08.2018			
        end
    end) ()
-------------------------------------------------
-- Панель корректировки времени 
-- Кнопка "следующее значение" - ">"
-------------------------------------------------

setTimeNextButton = CXBox:new()
    :bordercolor(cbubordoff)
    :fillcolor(cslidebofill)
    :textfont(smallfont)
    :textcolor(cbutextoff)
    :textpos("left")
    :movetext(-1, 10)
    :text(">")
    :attach(setTimeFrame, 167, 58, 7, 20)
    :visible(true)
    :active(true)

setTimeNextButton.Action = (function()
   return
      function()
         if toset ~= "" then -- added 13/08/2018 by Gironde
            if toset == "centiseconds" then
               date["seconds"] = date["seconds"]+0.01
               setTime()
            else
               date[toset] = date[toset]+1
               setTime()
            end
         end
      end
   end) ()	
------------------------------------------------- 
-- Панель корректировки времени 
-- Кнопка "100 единиц вперед" - ">>>"
-------------------------------------------------

setTimeNextButton = CXBox:new()
    :bordercolor(cbubordoff)
    :fillcolor(cslidebofill)
    :textfont(smallfont)
    :textcolor(cbutextoff)
    :textpos("left")
    :movetext(-1, 10)
    :text(">>>")
    :attach(setTimeFrame, 150, 5, 7, 73)

    :visible(true)
    :active(true)

setTimeNextButton.Action = (function()
    return
       function()
		 if toset ~= "" and toset == "year" then -- edit Gironde 13.08.2018
            date[toset] = date[toset]+100
            setTime()
		  end -- edit Gironde 12.08.2018
       end
    end) ()
	
-- ========================= Конец панели корректировки
-- ========================= Начало пиктограмм
-------------------------------------------------
-- Строка пиктограмм 
-- Реверс  времени - "<->"
-------------------------------------------------

revTimeButton = CXBox:new()   
    :bordercolor({1,1,0,0.5})  -- желтый   
    :fillimage(timeRevImage)    
    :visible(true)
    :active(true)  
    :attach(timeBox, 26*0+2, 3, 26*6+2, 32)

revTimeButton.Action = (function()
    return
        function()
             celestia:settimescale(-celestia:gettimescale())
        end
    end) ()

------------------------------------------------- 
-- Строка пиктограмм 
-- Замедление в 10 раз - "<<"
-------------------------------------------------
divTimeBy10Button = CXBox:new()
    :bordercolor(cbubordoff)
    :fillimage(timeDiv10Image)    
    :visible(true)
    :active(true) 
    :attach(timeBox, 26*1+2, 3, 26*5+2, 32)

divTimeBy10Button.Action = (function()
    return
        function()
            celestia:settimescale(celestia:gettimescale() / 10)
        end
    end) ()

-------------------------------------------------
-- Строка пиктограмм 
-- Замедление в 2 раза - "<<"
-------------------------------------------------
divTimeBy2Button = CXBox:new()
    :bordercolor(cbubordoff)
    :fillimage(timeDiv2Image)    
    :visible(true)
    :active(true)   
    :attach(timeBox, 26*2+2, 3, 26*4+2, 32)

divTimeBy2Button.Action = (function()
    return
        function()
            celestia:settimescale(celestia:gettimescale() / 2)
        end
    end) ()

-------------------------------------------------
-- Строка пиктограмм 
-- Переход в норм. режим времени - ">"
-------------------------------------------------
realTimeButton = CXBox:new()   
    :bordercolor({0,1,0,0.5})  -- зелёный 
    :fillimage(time1xImage)    
    :visible(true)
    :active(true)    
    :attach(timeBox, 26*3+2, 3, 26*3+2, 32)

realTimeButton.Action = (function()
    return
        function()
            display_realTime = not display_realTime
            if display_realTime then
               realTimeButton.Bordercolor = {1,0,0,.5}  -- красный
               celestia:settimescale(0)
            else
               realTimeButton.Bordercolor = {0,1,0,.5} -- зелёный   
               celestia:settimescale(1)        
            end
        end
    end) ()

-------------------------------------------------
-- Строка пиктограмм 
-- Ускорение в 2 раза - ">>"
-------------------------------------------------
multTimeBy2Button = CXBox:new()
    :bordercolor(cbubordoff)
    :fillimage(timeMult2Image)
    :visible(true)
    :active(true)
    :attach(timeBox, 26*4+2, 3, 26*2+2, 32)
 
multTimeBy2Button.Action = (function()
    return
        function()
            celestia:settimescale(celestia:gettimescale() * 2)
            if celestia:gettimescale() >= limitTimerate then
                celestia:settimescale(limitTimerate)
            end
        end
    end) ()

------------------------------------------------- 
-- Строка пиктограмм 
-- Ускорение в 10 раз - ">>>"
-------------------------------------------------
multTimeBy10Button = CXBox:new()
    :bordercolor(cbubordoff)
    :fillimage(timeMult10Image)    
    :visible(true)
    :active(true)
    :attach(timeBox, 26*5+2, 3, 26*1+2, 32)

multTimeBy10Button.Action = (function()
    return
        function()
            celestia:settimescale(celestia:gettimescale() * 10)
            if celestia:gettimescale() >= limitTimerate then
                celestia:settimescale(limitTimerate)
            end
        end
    end) ()

------------------------------------------------- 
-- Строка пиктограмм 
-- Текущее время - "Т"
-------------------------------------------------
currentTimeButton = CXBox:new()  
    :bordercolor({0,0.2,1,1})   
    :fillimage(timeCurImage)    
    :visible(true)
    :active(true) 
    :attach(timeBox, 26*6+2, 3, 26*0+2, 32)

currentTimeButton.Action = (function()
    return
        function()
            setCurrentTime()
        end
    end) ()

--==========================
-- перенесено из fovBox.lua
--==========================

require "locale"

----------------------------------------------
-- Set initial values
----------------------------------------------
yRatio = 722/height;
-- fovSliderFrameW = boxWidth - 24
fovSliderFrameW = 186-24
fovSliderBoxW = 0                                 
fovMidPos = (fovSliderFrameW - fovSliderBoxW) /2     

-- Set max/min values for zoom_speed.
zoom_speed = math.min(10, math.max(1, zoom_speed))

zoomToRad = 0.223832847;
fovCoeff = 2 / 3 * math.pi * zoomToRad -0.006           --Edit by Alexell
fov = celestia:getobserver():getfov()
zoom = fovCoeff / (yRatio * fov)
zoomFactor = 1e3 * 5 / zoom_speed
zoomPos = 0

-- Use the full range of FOV from 120 down to 3.6".
fovMin, fovMax = 0.001 * degToRad, 120 * degToRad

----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------
fovBox = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0, 0.5, 0.5, 0.5})
    :fillcolor(cFrameFill)
    :movable(false)
    :visible(true)
    :attach(panel7Frame, 0, 26*(gg-1), 0, 26*1+43)

fovBox.Customdraw =
    function(this)
        obs = celestia:getobserver()
        fov = obs:getfov()
        yRatio = 722 / height

        -- We use a simplified function to get Zoom from FOV; the function used in Celestiacore.cpp
        -- can't be reproduced here because there is no way to get certain variables from the core code.
        -- A tiny difference between the standard and the Lua values of Zoom may be noticed.
        zoom = fovCoeff / (yRatio * fov)

        textlayout:setpos(this.lb+5, this.tb-12+1)        
        textlayout:setfont(normalfont)     
        textlayout:setfontcolor(cbutextoff)
        textlayout:println(_("FOV:")..' '..format:raddms(fov)..' '..string.format("(%1.2fx)", zoom))

    end

fovSliderFrame = CXBox:new()
    :visible(true)
    :attach(fovBox, 5, 4, 5, 15)     
 
fovSliderFrame2 = CXBox:new()
    :bordercolor(cslidefrbord)
    :attach(fovSliderFrame, 0, 2, 0, 2)
    :visible(true)

fovSliderBox = CXBox:new()
    :bordercolor(cslidebobord)
    :fillcolor(cslidebofill)
    :visible(true)
    :movable(true)
    :attach(fovSliderFrame, fovMidPos, 0, fovMidPos, 0)

fovSliderFrame2.Masked = fovSliderBox

fovSliderBox.Customdraw =
    function(this)
        local obs = celestia:getobserver()
        fov = obs:getfov()
        if this.Move then
            local fov_lb = fovSliderFrame.lb
            local fov_lb2 = fovSliderBox.lb
            zoomPos = math.floor(fov_lb2 - fov_lb - fovMidPos)
            -- The slider has been moved; set the new FOV value.
            zoom = zoom * (1 + zoomPos / zoomFactor)
            fov = fovCoeff / (zoom * yRatio)
            if fov < fovMin then
                fov = fovMin
            elseif fov > fovMax then
                fov = fovMax
            end
            obs:setfov(fov)
        else
            fovSliderBox:attach(fovSliderFrame, fovMidPos, 0, fovMidPos, 0)
        end
    end;

resetFovButton = CXBox:new()
    :bordercolor(cbubordoff)
    --:textfont(normalfont)   
    :textfont(smallfont)
    :textcolor({1,0,0,1})
    :textpos("Left")
    :movetext(-2, 10)
    :text("A")
    :visible(true)
    :active(true)   
    :attach(fovBox, 170, 13, 6, 3)  
  
resetFovButton.Action = (function()
    return
        function()
            fov = 0.462012 / yRatio --Edit by Alexell
            obs:setfov(fov)
        end
    end) ()

--============================
-- перенесено из magnitudeBox.lua
--============================

require "locale";

----------------------------------------------
-- Define local functions
----------------------------------------------
local isAutoMagOn = function()
    autoMagOn = celestia:getrenderflags().automag
    return autoMagOn
end

local getMagSliderPos = function(magLevel, autoMagOn)
    if autoMagOn then
        magSliderPos = (magLevel-autoMagLevelMin)*magPosMax/(autoMagLevelMax-autoMagLevelMin)
    else
        magSliderPos = (magLevel-magLevelMin)*magPosMax/(magLevelMax-magLevelMin)
    end
    return magSliderPos
end

----------------------------------------------
-- Set initial values
----------------------------------------------
magLevel = celestia:getfaintestvisible()
autoMagOn = isAutoMagOn()

magSliderFrameW = 186 - 10
magSliderBoxW = 14

magSliderMargin = 3
magLevelMin = 1
magLevelMax = 15
autoMagLevelMin = 6
autoMagLevelMax = 12
magPosMax = magSliderFrameW-magSliderBoxW-(2*magSliderMargin)
magPos = getMagSliderPos(magLevel, autoMagOn)
magFirstTick = true

----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------
magnitudeBox = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0, 0.5, 0.5, 0.5})
    :fillcolor(cFrameFill)
    :movable(false) 
    :attach(panel7Frame, 0, 26*(gg-2), 0, 26*2+43)

magnitudeBox.Customdraw =
    function(this)
        local magLevel = celestia:getfaintestvisible()
        textlayout:setpos(this.lb+7-2, this.tb-15+2+2)
        textlayout:setfont(normalfont)
        textlayout:setfontcolor(cbutextoff) 
        magLevel_str = string.format("%1.2f", magLevel)
        autoMagOn = isAutoMagOn()
        if lastAutoMagOn ~= autoMagOn then
            lastAutoMagOn = autoMagOn
            magFirstTick = true
        end
        if autoMagOn then
            textlayout:println(_("AutoMag at 45°:")..' '..magLevel_str)
        else
            textlayout:println(_("Magnitude limit:")..' '..magLevel_str)
        end
    end

magSliderFrame = CXBox:new()
    :attach(magnitudeBox, 5, 4, 5, 15)    
    :visible(true)

magSliderFrame.Customdraw =
    function(this)
        mag_lb = magSliderFrame.lb
        mag_lb2 = magSliderBox.lb
        if autoMagOn then
            magValue = (mag_lb2-mag_lb-magSliderMargin)*(autoMagLevelMax-autoMagLevelMin)/magPosMax + autoMagLevelMin
        else
            magValue = (mag_lb2-mag_lb-magSliderMargin)*(magLevelMax-magLevelMin)/magPosMax + magLevelMin
        end
        if lastMagValue ~= magValue and not(magFirstTick) then
            -- The slider has been moved; set the new Magnitude limit/AutoMag at 45° value.
            lastMagValue = magValue
            celestia:setfaintestvisible(magValue)
        end
        magFirstTick = false
    end

magSliderFrame2 = CXBox:new()
    :bordercolor(cslidefrbord)
    :attach(magSliderFrame, 0, 2, 0, 2)
    :visible(true)

magSliderBox = CXBox:new()
    :bordercolor(cslidebobord)
    :fillcolor(cslidebofill)
    :visible(true)
    :movable(true)
    :attach(magSliderFrame, magSliderMargin+magPos, 0, magSliderFrameW-magSliderMargin-magPos-magSliderBoxW, 0)

magSliderFrame2.Masked = magSliderBox

magSliderBox.Customdraw =
    function(this)
        local magLevel = celestia:getfaintestvisible()
        if lastMagLevel ~= magLevel then
            -- Magnitude limit/AutoMag at 45° has been changed using the keyboard;
            -- set the new position of the slider.
            lastMagLevel = magLevel
            magPos = getMagSliderPos(magLevel, autoMagOn)
            magSliderBox:attach(magSliderFrame, magSliderMargin+magPos, 0, magSliderFrameW-magSliderMargin-magPos-magSliderBoxW, 0)
        end
    end

autoMagButton = CXBox:new()
    :bordercolor(cbubordoff)
    :textfont(normalfont)
    :textfont(smallfont)
    :textcolor({1,0,0,1}) 
    :textpos("left")
    :movetext(-2, 10)
    :text(_("A"))
    :visible(true)
    :active(true)

    :attach(magnitudeBox, 170, 13, 6, 5-2)

autoMagButton.Action = (function()
    return
        function()
            if isAutoMagOn() then
                celestia:setrenderflags{automag = false}
            else
                celestia:setrenderflags{automag = true}
            end
        end
    end) ()

--=========================
-- перенесено из galaxyLightBox.lua
--=========================
require "locale"
-------------------------------------------------
-- Set initial values
-------------------------------------------------
local galaxyLightLevel = celestia:getgalaxylightgain()
-- galaxyLightSliderFrameW = boxWidth - 16+7
galaxyLightSliderFrameW = 186 - 16+7
galaxyLightSliderBoxW = 12+3
galaxyLightSliderMargin = 3
galaxyLightPosMax = galaxyLightSliderFrameW-galaxyLightSliderBoxW-(2*galaxyLightSliderMargin)
galaxyLightPos = galaxyLightLevel*galaxyLightPosMax

----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------
galaxyLightBox = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0, 0.5, 0.5, 0.5}) 
    :fillcolor(cFrameFill)    
    :movable(false) 
    :attach(panel7Frame, 0, 26*(gg-3), 0, 26*3+43)

galaxyLightBox.Customdraw =
    function(this)
        local galaxyLightLevel = celestia:getgalaxylightgain()
        textlayout:setpos(this.lb+5, this.tb-12+1)     
        textlayout:setfont(normalfont)
        textlayout:setfontcolor(cbutextoff)
        galaxyLightLevel_str = string.format("%i%%", galaxyLightLevel*100)
        textlayout:println(_("Galaxy Light Gain:")..' '..galaxyLightLevel_str)
    end

galaxyLightSliderFrame = CXBox:new()
      :attach(galaxyLightBox, 5, 4, 5, 18-3)  
      :visible(true)

galaxyLightSliderFrame.Customdraw =
    function(this)
        mag_lb = galaxyLightSliderFrame.lb
        mag_lb2 = galaxyLightSliderBox.lb
        galaxyLightValue = math_round((mag_lb2-mag_lb-galaxyLightSliderMargin)*100/galaxyLightPosMax)/100
        if lastGalaxyLightValue ~= galaxyLightValue then
            -- The slider has been moved; set the new Galaxy Light gain value.
            -- Слайдер был перемещен; установить новое значение Galaxy Light.

            lastGalaxyLightValue = galaxyLightValue
            celestia:setgalaxylightgain(galaxyLightValue)
        end
    end

galaxyLightSliderFrame2 = CXBox:new()
    :bordercolor(cslidefrbord)
    :attach(galaxyLightSliderFrame, 0, 2, 0, 2) 
    :visible(true)

galaxyLightSliderBox = CXBox:new()
    :bordercolor(cslidebobord)
    :fillcolor(cslidebofill)
    :visible(true)
    :movable(true)
    :attach(galaxyLightSliderFrame, galaxyLightSliderMargin+galaxyLightPos, 0,
                        galaxyLightSliderFrameW-galaxyLightSliderMargin-galaxyLightPos-galaxyLightSliderBoxW, 0)

galaxyLightSliderFrame2.Masked = galaxyLightSliderBox

galaxyLightSliderBox.Customdraw =
    function(this)
        local galaxyLightLevel = celestia:getgalaxylightgain()
        if lastGalaxyLightLevel ~= galaxyLightLevel then
            -- Galaxy Light gain has been changed using the keyboard; Set the new position of the slider.
            -- Значение Galaxy Light было изменено с помощью клавиатуры; установить новое положение слайдера.
            
            lastGalaxyLightLevel = galaxyLightLevel
            galaxyLightPos = galaxyLightLevel * galaxyLightPosMax
            galaxyLightSliderBox:attach(galaxyLightSliderFrame, galaxyLightSliderMargin+galaxyLightPos, 0,
                                                                                            galaxyLightSliderFrameW-galaxyLightSliderMargin-galaxyLightPos-galaxyLightSliderBoxW, 0)
        end
    end

resetgalaxyLightSliderButton = CXBox:new()
    :bordercolor(cbubordoff)
    --:textfont(normalfont)   
    :textfont(smallfont)
    :textcolor({1,1,0,0.7})
    :textpos("Left")
    :movetext(-2, 10)
    :text("A")
    :visible(true)
    :active(true)  
  --  :attach(galaxyLightBox, 170, 14, 6, 5)  

--=========================
-- перенесено из lightBox.lua
--=========================

require "locale"
----------------------------------------------
-- Set initial values
----------------------------------------------
local lightLevel = celestia:getambient()
lightSliderFrameW = 186 - 9
lightSliderBoxW = 15
lightSliderMargin = 3
lightPosMax = lightSliderFrameW-lightSliderBoxW-(2*lightSliderMargin)
lightPos = lightLevel*lightPosMax
lightFirstTick = true

----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------
lightBox = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0, 0.5, 0.5, 0.5})   
    :fillcolor({0,0,0.5,.4}) 
    :fillcolor(cFrameFill)     
    :movable(false)   
    :attach(panel7Frame, 0, 26*(gg-4), 0, 26*4+43)

lightBox.Customdraw =
    function(this)
        local lightLevel = celestia:getambient()
        textlayout:setpos(this.lb+5, this.tb-12+1)
        textlayout:setfont(normalfont)       
        textlayout:setfontcolor(cbutextoff)
        lightLevel_str = string.format("%1.2f", lightLevel)
        textlayout:println(_("Ambient Light Level:")..' '..lightLevel_str)
    end

lightSliderFrame = CXBox:new()
    :visible(true)       
    :attach(lightBox, 5, 4, 5, 18-3)     

lightSliderFrame.Customdraw =
    function(this)
        light_lb = lightSliderFrame.lb
        light_lb2 = lightSliderBox.lb
        lightValue = math_round(light_lb2-light_lb-3)/lightPosMax
        if lastLightValue ~= lightValue and not(lightFirstTick) then
            -- The slider has been moved; set the new Ambient Light value.
            lastLightValue = lightValue
            celestia:setambient(lightValue)
        end
        lightFirstTick = false
    end

lightSliderFrame2 = CXBox:new()
    :bordercolor(cslidefrbord)
    :visible(true)
    :attach(lightSliderFrame, 0, 2, 0, 2)   

lightSliderBox = CXBox:new()
    :bordercolor(cslidebobord)
    :fillcolor(cslidebofill)
    :visible(true)
    :movable(true)
    :attach(lightSliderFrame, lightSliderMargin+lightPos, 0, lightSliderFrameW-lightSliderMargin-lightPos-lightSliderBoxW, 0)

lightSliderFrame2.Masked = lightSliderBox

lightSliderBox.Customdraw =
    function(this)
        local lightLevel = celestia:getambient()
        if lastLightLevel ~= lightLevel then
            -- Ambient Light Level has been changed using the keyboard
            -- set the new position of the slider.
            lastLightLevel = lightLevel
            lightPos = math_round(lightLevel*lightPosMax)

            lightSliderBox:attach(lightSliderFrame, lightSliderMargin+lightPos, 0, lightSliderFrameW-lightSliderMargin-lightPos-lightSliderBoxW, 0)
        end
    end